import doctest

import sortedcollections
import sortedcollections.ordereddict
import sortedcollections.recipes


def test_sortedcollections():
    failed, attempted = doctest.testmod(sortedcollections)
    assert attempted > 0
    assert failed == 0


def test_sortedcollections_ordereddict():
    failed, attempted = doctest.testmod(sortedcollections.ordereddict)
    assert attempted > 0
    assert failed == 0


def test_sortedcollections_recipes():
    failed, attempted = doctest.testmod(sortedcollections.recipes)
    assert attempted > 0
    assert failed == 0



"Test sortedcollections.recipes"

import pickle

import pytest

from sortedcollections import IndexableDict, IndexableSet, SegmentList


def test_index_dict():
    mapping = IndexableDict(enumerate(range(10)))
    iloc = mapping.keys()
    for value in range(10):
        assert iloc[value] == value


def test_index_set():
    set_values = IndexableSet(range(10))
    for index in range(10):
        assert set_values[index] == index


def test_index_set_pickle():
    set_values1 = IndexableSet(range(10))
    data = pickle.dumps(set_values1)
    set_values2 = pickle.loads(data)
    assert set_values1 == set_values2


def test_segment_list():
    values = [5, 1, 3, 2, 4, 8, 6, 7, 9, 0]
    sl = SegmentList(values)
    assert list(sl) == values
    sl.sort()
    assert list(sl) == list(range(10))
    sl.reverse()
    assert list(sl) == list(reversed(range(10)))
    sl.reverse()
    sl.append(10)
    assert list(sl) == list(range(11))
    sl.extend(range(11, 15))
    assert list(sl) == list(range(15))
    del sl[5:]
    assert list(sl) == list(range(5))
    sl[2] = 'c'
    sl.insert(3, 'd')
    sl.insert(6, 'e')
    assert list(sl) == [0, 1, 'c', 'd', 3, 4, 'e']


def test_segment_list_bisect():
    sl = SegmentList()
    with pytest.raises(NotImplementedError):
        sl.bisect(0)


def test_segment_list_setitem_slice():
    sl = SegmentList()
    with pytest.raises(NotImplementedError):
        sl[:] = [0]



"Test sortedcollections.OrderedSet."

import random

import pytest

from sortedcollections import OrderedSet


def test_init():
    os = OrderedSet()
    assert len(os) == 0


def test_contains():
    os = OrderedSet(range(100))
    assert len(os) == 100
    for value in range(100):
        assert value in os
        assert os.count(value) == 1
    assert -1 not in os
    assert 100 not in os


def test_iter():
    os = OrderedSet(range(100))
    assert list(os) == list(range(100))
    names = ['eve', 'carol', 'alice', 'dave', 'bob']
    os = OrderedSet(names)
    assert list(os) == names


def test_reversed():
    os = OrderedSet(range(100))
    assert list(reversed(os)) == list(reversed(range(100)))
    names = ['eve', 'carol', 'alice', 'dave', 'bob']
    os = OrderedSet(names)
    assert list(reversed(os)) == list(reversed(names))


def test_getitem():
    values = list(range(100))
    random.shuffle(values)
    os = OrderedSet(values)
    assert len(os) == len(values)
    for index in range(len(os)):
        assert os[index] == values[index]


def test_index():
    values = list(range(100))
    random.shuffle(values)
    os = OrderedSet(values)
    assert len(os) == len(values)
    for value in values:
        assert values.index(value) == os.index(value)


def test_index_error():
    os = OrderedSet(range(10))
    with pytest.raises(ValueError):
        os.index(10)


def test_add():
    os = OrderedSet()
    for value in range(100):
        os.add(value)
    assert len(os) == 100
    os.add(0)
    assert len(os) == 100
    for value in range(100):
        assert value in os


def test_discard():
    os = OrderedSet(range(100))
    for value in range(200):
        os.discard(value)
    assert len(os) == 0


def test_repr():
    os = OrderedSet()
    assert repr(os) == 'OrderedSet([])'



import pytest

from sortedcollections import NearestDict


def test_basic():
    d = NearestDict()

    with pytest.raises(KeyError):
        d[0]

    d[0] = 'a'
    assert d[0] == 'a'
    d[0] = 'b'
    assert d[0] == 'b'


def test_iteration():
    # In sorted order by key
    exp_items = (
        (0, 'a'),
        (1, 'b'),
        (2, 'c'),
    )

    d = NearestDict()
    for k, v in exp_items:
        d[k] = v

    for act, exp in zip(d.items(), exp_items):
        assert act == exp


def test_nearest():
    d = NearestDict(rounding=NearestDict.NEAREST)

    d[0] = 'a'
    d[3] = 'b'
    assert d[-1] == 'a'
    assert d[0] == 'a'
    assert d[1] == 'a'
    assert d[2] == 'b'
    assert d[3] == 'b'
    assert d[4] == 'b'


def test_nearest_prev():
    d = NearestDict(rounding=NearestDict.NEAREST_PREV)

    d[0] = 'a'
    d[3] = 'b'
    with pytest.raises(KeyError):
        d[-1]
    assert d[0] == 'a'
    assert d[1] == 'a'
    assert d[2] == 'a'
    assert d[3] == 'b'
    assert d[4] == 'b'


def test_nearest_next():
    d = NearestDict(rounding=NearestDict.NEAREST_NEXT)

    d[0] = 'a'
    d[3] = 'b'
    assert d[-1] == 'a'
    assert d[0] == 'a'
    assert d[1] == 'b'
    assert d[2] == 'b'
    assert d[3] == 'b'
    with pytest.raises(KeyError):
        d[4]



"Test sortedcollections.OrderedDict"

import pickle

import pytest

from sortedcollections import OrderedDict

pairs = dict(enumerate(range(10)))


def test_init():
    od = OrderedDict()
    assert len(od) == 0
    od._check()
    od = OrderedDict(enumerate(range(10)))
    assert len(od) == 10
    od._check()
    od = OrderedDict(a=0, b=1, c=2)
    assert len(od) == 3
    od._check()
    od = OrderedDict(pairs)
    assert len(od) == 10
    od._check()


def test_setitem():
    od = OrderedDict()
    od['alice'] = 0
    od['bob'] = 1
    od['carol'] = 2
    assert len(od) == 3
    od['carol'] = 2
    assert len(od) == 3
    od._check()


def test_delitem():
    od = OrderedDict(pairs)
    assert len(od) == 10
    for value in range(10):
        del od[value]
    assert len(od) == 0
    od._check()


def test_iter_reversed():
    od = OrderedDict([('b', 0), ('a', 1), ('c', 2)])
    assert list(od) == ['b', 'a', 'c']
    assert list(reversed(od)) == ['c', 'a', 'b']
    od._check()


def test_clear():
    od = OrderedDict(pairs)
    assert len(od) == 10
    od.clear()
    assert len(od) == 0
    od._check()


def test_popitem():
    od = OrderedDict(enumerate(range(10)))
    for num in reversed(range(10)):
        key, value = od.popitem()
        assert num == key == value
        od._check()

    od = OrderedDict(enumerate(range(10)))
    for num in range(10):
        key, value = od.popitem(last=False)
        assert num == key == value
        od._check()


def test_keys():
    od = OrderedDict(enumerate(range(10)))
    assert list(reversed(od.keys())) == list(reversed(range(10)))
    assert od.keys()[:3] == [0, 1, 2]
    od._check()


def test_items():
    items = list(enumerate(range(10)))
    od = OrderedDict(enumerate(range(10)))
    assert list(reversed(od.items())) == list(reversed(items))
    assert od.items()[:3] == [(0, 0), (1, 1), (2, 2)]
    od._check()


def test_values():
    od = OrderedDict(enumerate(range(10)))
    assert list(reversed(od.values())) == list(reversed(range(10)))
    assert od.values()[:3] == [0, 1, 2]
    od._check()


def test_iloc():
    od = OrderedDict(enumerate(range(10)))
    iloc = od.keys()
    for num in range(10):
        assert iloc[num] == num
    iloc[-1] == 9
    assert len(iloc) == 10
    od._check()


def test_pop():
    od = OrderedDict(enumerate(range(10)))
    for num in range(10):
        assert od.pop(num) == num
        od._check()
    assert od.pop(0, 'thing') == 'thing'
    assert od.pop(1, default='thing') == 'thing'
    od._check()


def test_pop_error():
    od = OrderedDict()
    with pytest.raises(KeyError):
        od.pop(0)


def test_setdefault():
    od = OrderedDict()
    od.setdefault(0, False)
    assert od[0] is False
    od.setdefault(1, default=True)
    assert od[1] is True
    od.setdefault(2)
    assert od[2] is None
    assert od.setdefault(0) is False
    assert od.setdefault(1) is True


def test_repr():
    od = OrderedDict()
    assert repr(od) == 'OrderedDict([])'
    assert str(od) == 'OrderedDict([])'


def test_reduce():
    od = OrderedDict(enumerate(range(10)))
    data = pickle.dumps(od)
    copy = pickle.loads(data)
    assert od == copy


def test_copy():
    od = OrderedDict(enumerate(range(10)))
    copy = od.copy()
    assert od == copy


def test_fromkeys():
    od = OrderedDict.fromkeys('abc')
    assert od == {'a': None, 'b': None, 'c': None}
    od._check()


def test_equality():
    od = OrderedDict.fromkeys('abc')
    assert od == {'a': None, 'b': None, 'c': None}
    assert od != {}
    assert od != OrderedDict()
    od._check()



import pathlib
import re

from setuptools import setup
from setuptools.command.test import test as TestCommand


class Tox(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import tox

        errno = tox.cmdline(self.test_args)
        exit(errno)


with open('README.rst') as reader:
    readme = reader.read()

init_text = (pathlib.Path('sortedcollections') / '__init__.py').read_text()
match = re.search(r"^__version__ = '(.+)'$", init_text, re.MULTILINE)
version = match.group(1)

setup(
    name='sortedcollections',
    version=version,
    description='Python Sorted Collections',
    long_description=readme,
    author='Grant Jenks',
    author_email='contact@grantjenks.com',
    url='http://www.grantjenks.com/docs/sortedcollections/',
    license='Apache 2.0',
    packages=['sortedcollections'],
    install_requires=['sortedcontainers'],
    tests_require=['tox'],
    cmdclass={'test': Tox},
    classifiers=(
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
    ),
)



"Test sortedcollections.ItemSortedDict"

import copy

import pytest

from sortedcollections import ItemSortedDict


def key_func(key, value):
    return key


def value_func(key, value):
    return value


alphabet = 'abcdefghijklmnopqrstuvwxyz'


def test_init():
    temp = ItemSortedDict(key_func)
    temp._check()


def test_init_args():
    temp = ItemSortedDict(key_func, enumerate(alphabet))
    assert len(temp) == 26
    assert temp[0] == 'a'
    assert temp[25] == 'z'
    assert temp.keys()[4] == 4
    temp._check()


def test_init_kwargs():
    temp = ItemSortedDict(key_func, a=0, b=1, c=2)
    assert len(temp) == 3
    assert temp['a'] == 0
    assert temp.keys()[0] == 'a'
    temp._check()


def test_getitem():
    temp = ItemSortedDict(value_func, enumerate(reversed(alphabet)))
    assert temp[0] == 'z'
    assert temp.keys()[0] == 25
    assert list(temp) == list(reversed(range(26)))


def test_delitem():
    temp = ItemSortedDict(value_func, enumerate(reversed(alphabet)))
    del temp[25]
    assert temp.keys()[0] == 24


def test_delitem_error():
    temp = ItemSortedDict(value_func, enumerate(reversed(alphabet)))
    with pytest.raises(KeyError):
        del temp[-1]


def test_setitem():
    temp = ItemSortedDict(value_func, enumerate(reversed(alphabet)))
    temp[25] = '!'
    del temp[25]
    iloc = temp.keys()
    assert iloc[0] == 24
    temp[25] = 'a'
    assert iloc[0] == 25


def test_copy():
    temp = ItemSortedDict(value_func, enumerate(reversed(alphabet)))
    that = temp.copy()
    assert temp == that
    assert temp._key != that._key


def test_deepcopy():
    temp = ItemSortedDict(value_func, enumerate(reversed(alphabet)))
    that = copy.deepcopy(temp)
    assert temp == that
    assert temp._key != that._key


def test_update():
    temp = ItemSortedDict(lambda key, value: value)
    for index, letter in enumerate(alphabet):
        pair = {index: letter}
        temp.update(pair)
